@@ -12,7 +12,6 @@ import importlib
 import inspect
 import itertools
 import re
-import pydoc
 import sys
 import types
 import typing
@@ -24,12 +23,17 @@ from argparse import (
 from collections import defaultdict, namedtuple, Counter
 from enum import Enum
 from pathlib import PurePath
+from types import MethodType
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 try:
     import importlib.metadata as _im
 except ImportError:
     import importlib_metadata as _im
+try:
+    from pkgutil import resolve_name as _pkgutil_resolve_name
+except ImportError:
+    from pkgutil_resolve_name import resolve_name as _pkgutil_resolve_name
 try:
     from typing import Annotated
 except ImportError:
@@ -63,10 +67,16 @@ try:
 except ImportError:
     __version__ = '0+unknown'
 
-__all__ = ['run', 'signature', 'bind']
+__all__ = ['run', 'signature', 'bind', 'bind_known']
 
 _PARAM_TYPES = ['param', 'parameter', 'arg', 'argument', 'key', 'keyword']
 _TYPE_NAMES = ['type', 'kwtype']
+_LIST_TYPES = [
+    list,
+    collections.abc.Iterable,
+    getattr(collections.abc, "Collection", object()),
+    collections.abc.Sequence,
+]
 
 _Doc = namedtuple('_Doc', ('first_line', 'text', 'params', 'raises'))
 _Param = namedtuple('_Param', ('text', 'type'))
@@ -139,77 +149,148 @@ class _DefaultList(list):
     """
 
 
+def _check_in_list(_values, **kwargs):
+    for k, v in kwargs.items():
+        if v not in _values:
+            raise ValueError(
+                '{!r} must be one of {!r}, not {!r}'.format(k, _values, v))
+
+
+def _extract_raises(sig):
+    raises, = [  # typing_inspect does not allow fetching metadata, e.g. ti#82.
+        arg for arg in getattr(sig.return_annotation, '__metadata__', [])
+        if isinstance(arg, _Raises)]
+    return raises
+
+
 _unset = 'UNSET'
 
 
-def bind(funcs: Union[Callable, List[Callable], Dict[str, Callable]], *,
-         parsers: Dict[type, Callable[[str], Any]] = {},
-         short: Optional[Dict[str, str]] = None,
-         cli_options: Literal['kwonly', 'all', 'has_default'] = _unset,
-         strict_kwonly=_unset,
-         show_defaults: bool = True,
-         show_types: bool = False,
-         no_negated_flags: bool = False,
-         version: Union[str, None, bool] = None,
-         argparse_kwargs: dict = {},
-         argv: Optional[List[str]] = None):
-    """
-    Process command-line arguments and bind arguments.
-
-    This function takes the same parameters as `defopt.run`, but returns a
-    pair, which consists of a `~typing.Callable` *func* and a
-    `~inspect.BoundArguments` *ba*, such that `defopt.run` would call
-    ``func(*ba.args, **ba.kwargs)`` (modulo exception handling).
-    """
+def _bind_or_bind_known(
+    funcs: Union[Callable, List[Callable], Dict[str, Callable]], *,
+    parsers: Dict[type, Callable[[str], Any]] = {},
+    short: Optional[Dict[str, str]] = None,
+    cli_options: Literal['kwonly', 'all', 'has_default'] = _unset,
+    strict_kwonly=_unset,
+    show_defaults: bool = True,
+    show_types: bool = False,
+    no_negated_flags: bool = False,
+    version: Union[str, None, bool] = None,
+    argparse_kwargs: dict = {},
+    intermixed: bool = False,
+    _known: bool = False,
+    argv: Optional[List[str]] = None):
     if strict_kwonly == _unset:
         if cli_options == _unset:
             cli_options = 'kwonly'
     else:
         if cli_options != _unset:
             raise ValueError(
-                "Cannot pass both 'cli_options' and 'strict_kwonly'")
+                "cannot pass both 'cli_options' and 'strict_kwonly'")
         warnings.warn(
             'strict_kwonly is deprecated and will be removed in an upcoming '
             'release', DeprecationWarning)
         cli_options = 'kwonly' if strict_kwonly else 'has_default'
+    _check_in_list(['kwonly', 'all', 'has_default'], cli_options=cli_options)
     parser = _create_parser(
         funcs, parsers=parsers, short=short, cli_options=cli_options,
         show_defaults=show_defaults, show_types=show_types,
         no_negated_flags=no_negated_flags, version=version,
         argparse_kwargs=argparse_kwargs)
     with _colorama_text():
-        parsed_argv = vars(parser.parse_args(argv))
+        if not intermixed:
+            if not _known:
+                args, rest = parser.parse_args(argv), []
+            else:
+                args, rest = parser.parse_known_args(argv)
+        else:
+            if sys.version_info < (3, 7):
+                raise RuntimeError("'intermixed' requires Python>=3.7")
+            if not _known:
+                args, rest = parser.parse_intermixed_args(argv), []
+            else:
+                args, rest = parser.parse_known_intermixed_args(argv)
+    parsed_args = vars(args)
     try:
-        func = parsed_argv.pop('_func')
+        func = parsed_args.pop('_func')
     except KeyError:
         # Workaround for http://bugs.python.org/issue9253#msg186387 (and
         # https://bugs.python.org/issue29298 which blocks using required=True).
         parser.error('too few arguments')
     sig = signature(func)
     ba = sig.bind_partial()
-    ba.arguments.update(parsed_argv)
-    return func, ba
+    ba.arguments.update(parsed_args)
+    call = functools.partial(func, *ba.args, **ba.kwargs)
+    raises = _extract_raises(sig)
+
+    @functools.wraps(call)
+    def wrapper() -> sig.return_annotation:
+        try:
+            return call()
+        except raises as e:
+            sys.exit(e)
+
+    return wrapper, rest
 
 
-def run(funcs: Union[Callable, List[Callable], Dict[str, Callable]], *,
-        parsers: Dict[type, Callable[[str], Any]] = {},
-        short: Optional[Dict[str, str]] = None,
-        cli_options: Literal['kwonly', 'all', 'has_default'] = _unset,
-        strict_kwonly=_unset,
-        show_defaults: bool = True,
-        show_types: bool = False,
-        no_negated_flags: bool = False,
-        version: Union[str, None, bool] = None,
-        argparse_kwargs: dict = {},
-        argv: Optional[List[str]] = None):
+def bind(*args, **kwargs):
+    """
+    Process command-line arguments and bind arguments.
+
+    This function takes the same parameters as `defopt.run`, but returns a
+    `~functools.partial` object which represents the call that ``defopt.run``
+    would execute.  In other words, ``call = defopt.bind(...); call()`` is
+    equivalent to ``defopt.run(...)``.  Note that no argument needs to be
+    passed explicitly to ``call``; everything is handled internally.
+
+    The ``call.func`` attribute is a thin wrapper around one of the functions
+    passed to `bind` (to suppress documented raisable exceptions).  The
+    original function is accessible as ``call.func.__wrapped__``.
+
+    The ``call.args`` and ``call.keywords`` attributes are set according to the
+    command-line arguments.
+
+    This API is provisional and may be adjusted depending on feedback.
+    """
+    call, rest = _bind_or_bind_known(*args, _known=False, **kwargs)
+    assert not rest
+    return call
+
+
+def bind_known(*args, **kwargs):
+    """
+    Process command-line arguments and bind known arguments.
+
+    This function behaves as `bind`, but returns a pair of 1) the
+    `~functools.partial` callable, and 2) a list of unknown command-line
+    arguments, as returned by `~argparse.ArgumentParser.parse_known_args`.
+
+    This API is provisional and may be adjusted depending on feedback.
+    """
+    return _bind_or_bind_known(*args, _known=True, **kwargs)
+
+
+def run(
+    funcs: Union[Callable, List[Callable], Dict[str, Callable]], *,
+    parsers: Dict[type, Callable[[str], Any]] = {},
+    short: Optional[Dict[str, str]] = None,
+    cli_options: Literal['kwonly', 'all', 'has_default'] = _unset,
+    strict_kwonly=_unset,
+    show_defaults: bool = True,
+    show_types: bool = False,
+    no_negated_flags: bool = False,
+    version: Union[str, None, bool] = None,
+    argparse_kwargs: dict = {},
+    intermixed: bool = False,
+    argv: Optional[List[str]] = None):
     """
     Process command-line arguments and run the given functions.
 
-    ``funcs`` can be a single callable, which is parsed and run; or it can
-    be a list of callables or mappable of strs to callables, in which case
-    each one is given a subparser with its name (if ``funcs`` is a list) or
-    the corresponding key (if ``funcs`` is a mappable), and only the chosen
-    callable is run.
+    *funcs* can be a single callable, which is parsed and run; or it can be
+    a list of callables or mappable of strs to callables, in which case each
+    one is given a subparser with its name (if *funcs* is a list) or the
+    corresponding key (if *funcs* is a mappable), and only the chosen callable
+    is run.
 
     :param funcs:
         Function or functions to process and run.
@@ -255,26 +336,24 @@ def run(funcs: Union[Callable, List[Callable], Dict[str, Callable]], *,
     :param argparse_kwargs:
         A mapping of keyword arguments that will be passed to the
         `~argparse.ArgumentParser` constructor.
+    :param intermixed:
+        Whether to use `~argparse.ArgumentParser.parse_intermixed_args` to
+        parse the command line.  Intermixed parsing imposes many restrictions,
+        listed in the `argparse` documentation.
     :param argv:
         Command line arguments to parse (default: ``sys.argv[1:]``).
     :return:
         The value returned by the function that was run.
     """
-    func, ba = bind(
+    call = bind(
         funcs, parsers=parsers, short=short, cli_options=cli_options,
         strict_kwonly=strict_kwonly, show_defaults=show_defaults,
         show_types=show_types, no_negated_flags=no_negated_flags,
-        version=version, argparse_kwargs=argparse_kwargs, argv=argv)
-    sig = signature(func)
-    raises, = [
-        # typing_inspect does not allow fetching metadata; see e.g. ti#82.
-        arg for arg in getattr(sig.return_annotation, '__metadata__', [])
-        if isinstance(arg, _Raises)]
-    # The function call should occur here to minimize effects on the traceback.
-    try:
-        return func(*ba.args, **ba.kwargs)
-    except raises as e:
-        sys.exit(e)
+        version=version, argparse_kwargs=argparse_kwargs,
+        intermixed=intermixed, argv=argv)
+    if not _extract_raises(inspect.signature(call, follow_wrapped=False)):
+        call = call.__wrapped__  # Suppress a trivial traceback frame.
+    return call()
 
 
 def _recurse_functions(funcs, subparsers):
@@ -286,8 +365,8 @@ def _recurse_functions(funcs, subparsers):
         except AttributeError as exc:
             # Do not allow a mapping inside of a list
             raise ValueError(
-                'Use dictionaries (mappings) for nesting; other iterables may '
-                'only contain functions (callables).'
+                'use dictionaries (mappings) for nesting; other iterables may '
+                'only contain functions (callables)'
             ) from exc
 
     for name, func in funcs.items():
@@ -335,7 +414,7 @@ def _create_parser(
     elif version is None or version:
         version_string = _get_version(version_sources)
         if version and version_string is None:
-            raise ValueError('Failed to autodetect version string')
+            raise ValueError('failed to autodetect version string')
     else:
         version_string = None
     if version_string is not None:
@@ -400,23 +479,15 @@ def signature(func: Callable):
     - The return type is `~typing.Annotated` with the documented raisable
       exception types, in wrapped in a private tuple subclass.
     """
-    return _signature(func)
-
-
-def _signature(func, *, skip_first_arg=False):
-    # See _is_constructible_from_str for skip_first_arg.  We can't just drop
-    # the first arg later because it may be unannotated.
     orig_sig = inspect.signature(func)
     orig_params = orig_sig.parameters.values()
-    if skip_first_arg:
-        _, *orig_params = orig_params
     doc = _parse_docstring(inspect.getdoc(func))
     parameters = []
     for param in orig_params:
         if param.name.startswith('_'):
             if param.default is param.empty:
                 raise ValueError(
-                    'Parameter {} of {}{} is private but has no default'
+                    'parameter {} of {}{} is private but has no default'
                     .format(param.name, func.__name__, orig_sig))
         else:
             parameters.append(Parameter(
@@ -441,6 +512,7 @@ def _populate_parser(func, parser, parsers, short, cli_options,
         if ((cli_options == 'kwonly' or
              (param.default is param.empty and cli_options == 'has_default'))
             and not _is_list_like(param.annotation)
+            and not _is_optional_list_like(param.annotation)
             and param.kind != param.KEYWORD_ONLY)}
     if short is None:
         count_initials = Counter(name[0] for name in sig.parameters
@@ -463,8 +535,8 @@ def _populate_parser(func, parser, parsers, short, cli_options,
         required = not hasdefault and param.kind != param.VAR_POSITIONAL
         positional = name in positionals
         if type_ in [bool, typing.Optional[bool]] and not positional:
-            action = ('store_true' if no_negated_flags and
-                      default in [False, None]
+            action = ('store_true'
+                      if no_negated_flags and default in [False, None]
                       else _BooleanOptionalAction)  # --name/--no-name
             actions.append(_add_argument(
                 parser, name, short, action=action, default=default,
@@ -484,6 +556,16 @@ def _populate_parser(func, parser, parsers, short, cli_options,
                 kwargs['default'] = _DefaultList()
         else:
             kwargs['required'] = required
+
+        # If the type is an Optional container, extract only the container.
+        union_args = _ti_get_args(type_) if _is_union_type(type_) else []
+        if any(_is_container(subtype) for subtype in union_args):
+            non_none = [arg for arg in union_args if arg is not type(None)]
+            if len(non_none) != 1:
+                raise ValueError('unsupported union including container type: '
+                                 '{}'.format(type_))
+            type_, = non_none
+
         if _is_list_like(type_):
             type_, = _ti_get_args(type_)
             kwargs['nargs'] = '*'
@@ -491,16 +573,25 @@ def _populate_parser(func, parser, parsers, short, cli_options,
                 kwargs['action'] = 'append'
                 kwargs['default'] = _DefaultList()
         member_types = None
-        if _ti_get_origin(type_) is tuple:
+
+        if isinstance(type_, type) and issubclass(type_, Enum):
+            # Enums must be checked first to handle enums-of-namedtuples.
+            kwargs['type'] = _get_parser(type_, parsers)
+            kwargs['metavar'] = '{' + ','.join(type_.__members__) + '}'
+        elif _ti_get_origin(type_) is tuple:
             member_types = _ti_get_args(type_)
+            num_members = len(member_types)
             # Variable-length tuples of homogenous type are specified like
             # Tuple[int, ...]
-            if len(member_types) == 2 and member_types[1] is Ellipsis:
-                kwargs['nargs'] = "*"
+            if num_members == 2 and member_types[1] is Ellipsis:
+                kwargs['nargs'] = '*'
                 kwargs['action'] = _make_store_tuple_action_class(
                     tuple, member_types, parsers, is_variable_length=True)
+            elif type(None) in union_args and parsers.get(type(None)):
+                raise ValueError('Optional tuples and NoneType parsers cannot '
+                                 'be used together due to ambiguity')
             else:
-                kwargs['nargs'] = len(member_types)
+                kwargs['nargs'] = num_members
                 kwargs['action'] = _make_store_tuple_action_class(
                     tuple, member_types, parsers)
         elif (isinstance(type_, type) and issubclass(type_, tuple)
@@ -516,9 +607,7 @@ def _populate_parser(func, parser, parsers, short, cli_options,
                 kwargs['metavar'] = type_._fields
         else:
             kwargs['type'] = _get_parser(type_, parsers)
-            if isinstance(type_, type) and issubclass(type_, Enum):
-                kwargs['metavar'] = '{' + ','.join(type_.__members__) + '}'
-            elif _ti_get_origin(type_) is Literal:
+            if _ti_get_origin(type_) is Literal:
                 kwargs['metavar'] = (
                     '{' + ','.join(map(str, _ti_get_args(type_))) + '}')
         actions.append(_add_argument(parser, name, short, **kwargs))
@@ -577,12 +666,23 @@ def _update_help_string(action, *, show_defaults, show_types):
 
 
 def _is_list_like(type_):
-    return _ti_get_origin(type_) in [
-        list,
-        collections.abc.Iterable,
-        getattr(collections.abc, 'Collection', object()),
-        collections.abc.Sequence,
-    ]
+    return _ti_get_origin(type_) in _LIST_TYPES
+
+
+def _is_container(type_):
+    return _ti_get_origin(type_) in {*_LIST_TYPES, tuple}
+
+
+def _is_union_type(type_):
+    return _ti_get_origin(type_) in {Union, getattr(types, 'UnionType', '')}
+
+
+def _is_optional_list_like(type_):
+    # Assume a union with a list subtype is actually Optional[list[...]]
+    # because this condition is enforced in other places
+    return (_is_union_type(type_)
+            and any(_is_list_like(subtype) for subtype in _ti_get_args(type_)))
+
 
 def _get_type(func, name):
     """
@@ -623,18 +723,12 @@ def _get_type_from_doc(name, globalns):
     if ' or ' in name:
         subtypes = [_get_type_from_doc(part, globalns)
                     for part in name.split(' or ')]
-        if any(map(_is_list_like, subtypes)):
+        if any(map(_is_list_like, subtypes)) and None not in subtypes:
             raise ValueError(
                 'unsupported union including container type: {}'.format(name))
         return Union[tuple(subtype for subtype in subtypes)]
-    # Support for sphinx-specific "list[type]", "tuple[type]" syntax; only
-    # needed for Py<3.9.
-    # (This intentionally won't catch `List` or `typing.List`.)
-    match = re.match(r'(list|tuple)\[([\w\.]+)\]', name)
-    if match:
-        container, type_ = match.groups()
-        container = {'list': List, 'tuple': Tuple}[container]
-        return container[eval(type_, globalns)]
+    if sys.version_info < (3, 9):  # Support "list[type]", "tuple[type]".
+        globalns = {**globalns, 'tuple': Tuple, 'list': List}
     return _get_type_from_hint(eval(name, globalns))
 
 
@@ -642,15 +736,6 @@ def _get_type_from_hint(hint):
     if _is_list_like(hint):
         [type_] = _ti_get_args(hint)
         return List[type_]
-    elif _ti_get_origin(hint) is Union:
-        args = _ti_get_args(hint)
-        if any(_is_list_like(subtype) for subtype in args):
-            non_none = [arg for arg in args if arg is not type(None)]
-            if len(non_none) != 1:
-                raise ValueError(
-                    'unsupported union including container type: {}'
-                    .format(hint))
-            return non_none[0]
     return hint
 
 
@@ -742,6 +827,18 @@ def _parse_docstring(doc):
 
         depart_Text = _do_nothing
 
+        visit_reference = depart_reference = _do_nothing
+
+        def visit_target(self, node):
+            if self._current_paragraph is None:
+                raise SkipNode
+
+            if node.get('refuri'):
+                self._current_paragraph.append(" ({})".format(node['refuri']))
+            else:
+                self._current_paragraph.append(node.astext())
+            raise SkipNode
+
         def visit_emphasis(self, node):
             self._current_paragraph.append('\033[3m')  # *foo*: italic
 
@@ -894,7 +991,7 @@ def _get_parser(type_, parsers):
         parser = _make_enum_parser(type_)
     elif _is_constructible_from_str(type_):
         parser = functools.partial(type_)
-    elif _ti_get_origin(type_) in [Union, getattr(types, "UnionType", "")]:
+    elif _is_union_type(type_):
         args = _ti_get_args(type_)
         if type(None) in args:
             # If None is in the Union, parse it first.  This only matters if
@@ -927,7 +1024,7 @@ def _parse_bool(string):
     elif string.lower() in ['f', 'false', '0']:
         return False
     else:
-        raise ValueError('{} is not a valid boolean string'.format(string))
+        raise ValueError('{!r} is not a valid boolean string'.format(string))
 
 
 def _parse_slice(string):
@@ -944,12 +1041,12 @@ def _parse_slice(string):
         SliceVisitor().visit(ast.parse('_[{}]'.format(string)))
         sl, = slices
     except (SyntaxError, ValueError):
-        raise ValueError('{} is not a valid slice string'.format(string))
+        raise ValueError('{!r} is not a valid slice string'.format(string))
     return sl
 
 
 def _parse_none(string):
-    raise ValueError('No string can be converted to None')
+    raise ValueError('no string can be converted to None')
 
 
 def _make_enum_parser(enum, value=None):
@@ -963,26 +1060,26 @@ def _make_enum_parser(enum, value=None):
                 value, ', '.join(map(repr, enum.__members__))))
 
 
-def _is_constructible_from_str(type_, *, skip_first_arg=False):
+def _is_constructible_from_str(type_):
     try:
-        sig = _signature(type_, skip_first_arg=skip_first_arg)
+        sig = signature(type_)
         (argname, _), = sig.bind(object()).arguments.items()
     except TypeError:  # Can be raised by signature() or Signature.bind().
         return False
     except ValueError:
-        # Can be raised for classes, if the relevant info is in `__init__`.
-        if not isinstance(type_, type):
-            raise
+        # No relevant info in signature; continue below to also look in
+        # `type_.__init__`, in the case where type_ is indeed a type.
+        pass
     else:
         if sig.parameters[argname].annotation is str:
             return True
     if isinstance(type_, type):
         # signature() first checks __new__, if it is present.
-        # skip_first_arg behaves as if the first parameter was bound, i.e.
-        # __init__.__get__(object(), type_) but the latter can fail for types
-        # implemented in C (which don't support binding arbitrary objects).
-        return _is_constructible_from_str(
-            type_.__init__, skip_first_arg=True)
+        # `MethodType(type_.__init__, object())` binds the first parameter of
+        # `__init__` -- similarly to `__init__.__get__(object(), type_)`, but
+        # the latter can fail for types implemented in C (which may not support
+        # binding arbitrary objects).
+        return _is_constructible_from_str(MethodType(type_.__init__, object()))
     return False
 
 
@@ -1003,11 +1100,10 @@ def _make_literal_parser(literal, parsers, value=None):
         return functools.partial(_make_literal_parser, literal, parsers)
     for arg, p in zip(_ti_get_args(literal), parsers):
         try:
-            parsed = p(value)
+            if p(value) == arg:
+                return arg
         except ValueError:
             pass
-        if parsed == arg:
-            return arg
     raise ArgumentTypeError(
         'invalid choice: {!r} (choose from {})'.format(
             value, ', '.join(map(repr, map(str, _ti_get_args(literal))))))
@@ -1036,15 +1132,12 @@ def _make_store_tuple_action_class(
 if __name__ == '__main__':
     def main(argv=None):
         parser = ArgumentParser()
-        parser.add_argument('function')
+        parser.add_argument(
+            'function',
+            help='package.name.function_name or package.name:function_name')
         parser.add_argument('args', nargs=REMAINDER)
         args = parser.parse_args(argv)
-        try:
-            func = pydoc.locate(args.function)
-        except pydoc.ErrorDuringImport as exc:
-            raise exc.value from None
-        if func is None:
-            raise ImportError('Failed to locate {!r}'.format(args.function))
+        func = _pkgutil_resolve_name(args.function)
         argparse_kwargs = (
             {'prog': ' '.join(sys.argv[:2])} if argv is None else {})
         retval = run(func, argv=args.args, argparse_kwargs=argparse_kwargs)
