@@ -1,18 +1,20 @@
 import builtins
 import contextlib
 import inspect
+import multiprocessing as mp
 import re
+import runpy
 import subprocess
 import sys
 import tokenize
 import types
 import typing
 import unittest
+from concurrent.futures import ProcessPoolExecutor
 from contextlib import ExitStack
 from enum import Enum
 from io import StringIO
 from pathlib import Path
-from tempfile import TemporaryDirectory
 
 import defopt
 from defopt import __version__
@@ -77,13 +79,10 @@ class TestDefopt(unittest.TestCase):
     def test_nested_lists_invalid(self):
         def sub1(*bar):
             """:type bar: float"""
-            return bar
         def subsub1(*, baz=None):
             """:type baz: int"""
-            return baz
         def subsub2(*foo):
             """:type foo: float"""
-            return foo
         with self.assertRaises(ValueError):
             defopt.run([sub1, [subsub1, subsub2]], argv=['sub1', '1.2'])
 
@@ -164,13 +163,10 @@ class TestDefopt(unittest.TestCase):
     def test_nested_subcommands_mixed_invalid1(self):
         def sub1(*bar):
             """:type bar: float"""
-            return bar
         def subsub1(*, baz=None):
             """:type baz: int"""
-            return baz
         def subsub2(*foo):
             """:type foo: float"""
-            return foo
         with self.assertRaises(ValueError):
             defopt.run([sub1, {'sub2': [subsub1, subsub2]}],
                         argv=['sub1', '1.2'])
@@ -184,13 +180,10 @@ class TestDefopt(unittest.TestCase):
     def test_nested_subcommands_mixed_invalid2(self):
         def sub(*bar):
             """:type bar: float"""
-            return bar
         def subsub_with_dash(*, baz=None):
             """:type baz: int"""
-            return baz
         def subsub(*foo):
             """:type foo: float"""
-            return foo
         with self.assertRaises(ValueError):
             defopt.run([sub, {'subsub1': subsub_with_dash, 'subsub2': subsub}],
                        argv=['sub', '1.2'])
@@ -287,7 +280,7 @@ class TestDefopt(unittest.TestCase):
         def main(_a: int): assert False
         with self.assertRaisesRegex(ValueError,
                                     # Older Pythons have no space post-colon.
-                                    r'Parameter _a of main\(_a: ?int\) is '
+                                    r'parameter _a of main\(_a: ?int\) is '
                                     r'private but has no default'):
             defopt.run(main, argv=[])
 
@@ -300,6 +293,30 @@ class TestDefopt(unittest.TestCase):
                        argv=['+a', 'foo']),
             'foo')
 
+    def test_intermixed(self):
+        def main(*args: int, key: str): return args, key
+        with self.assertRaises(SystemExit):
+            defopt.run(main, argv=['1', '-kk', '2'])
+        if sys.version_info >= (3, 7):
+            self.assertEqual(
+                defopt.run(main, argv=['1', '-kk', '2'], intermixed=True),
+                ((1, 2), 'k'))
+
+
+class TestBindKnown(unittest.TestCase):
+    def test_bind_known(self):
+        def main(*args: int, key: str): return args, key
+        call, rest = defopt.bind_known(
+            main, argv=['1', '2', '-kk', '-qq'])
+        self.assertEqual((call(), rest), (((1, 2), 'k'), ['-qq']))
+        call, rest = defopt.bind_known(
+            main, argv=['1', '-kk', '2', '-qq'])
+        self.assertEqual((call(), rest), (((1,), 'k'), ['2', '-qq']))
+        if sys.version_info >= (3, 7):
+            call, rest = defopt.bind_known(
+                main, argv=['1', '-kk', '2', '-qq'], intermixed=True)
+            self.assertEqual((call(), rest), (((1, 2), 'k'), ['-qq']))
+
 
 class TestParsers(unittest.TestCase):
     def test_parser(self):
@@ -574,7 +591,7 @@ class TestParsers(unittest.TestCase):
         def notok(foo):
             """:type foo: NotConstructibleFromStr"""
 
-        with self.assertRaises(Exception):
+        with self.assertRaisesRegex(Exception, 'no parser.*NotConstructible'):
             defopt.run(notok, argv=["foo"])
 
 
@@ -685,6 +702,14 @@ class TestTuple(unittest.TestCase):
         self.assertEqual(str(defopt.run(main, argv=['1', '2', '3'])),
                          str(Pair(1, '2')))
 
+    def test_enumnamedtuple(self):
+        class EnumPair(Pair, Enum):
+            a = Pair('A', 1)
+            b = Pair('B', 2)
+
+        def main(foo: EnumPair): return foo
+        self.assertEqual(defopt.run(main, argv=['a']), EnumPair.a)
+
     def test_tuple_fails_early(self):
         def main(foo):
             """:param typing.Tuple[int,EmptyType] foo: foo"""
@@ -741,6 +766,93 @@ class TestUnion(unittest.TestCase):
         self.assertEqual(defopt.run(main, argv=['--foo', '1']), [1])
 
 
+class TestOptional(unittest.TestCase):
+    def test_optional_hint_list(self):
+        def main(op: typing.Optional[typing.List[str]]): return op
+        self.assertEqual(defopt.run(main, argv=['--op', '1', '2']), ['1', '2'])
+
+    def test_optional_hint_tuple(self):
+        def main(op: typing.Optional[typing.Tuple[int]]): return op
+        self.assertEqual(defopt.run(main, argv=['1']), (1,))
+
+    def test_optional_doc_list(self):
+        def main(op):
+            """:param typing.Optional[typing.List[int]] op: op"""
+            return op
+        self.assertEqual(defopt.run(main, argv=['--op', '1', '2']), [1, 2])
+
+    def test_optional_doc_tuple(self):
+        def main(op):
+            """:param typing.Optional[typing.Tuple[str]] op: op"""
+            return op
+        self.assertEqual(defopt.run(main, argv=['1']), ('1',))
+
+    def test_union_operator_hint_list(self):
+        if not hasattr(types, "UnionType"):
+            raise unittest.SkipTest("A|B-style unions not supported")
+        def main(op: typing.List[str] | None): return op
+        self.assertEqual(defopt.run(main, argv=['--op', '1', '2']), ['1', '2'])
+
+    def test_union_operator_hint_tuple(self):
+        if not hasattr(types, "UnionType"):
+            raise unittest.SkipTest("A|B-style unions not supported")
+        def main(op: typing.Tuple[int] | None): return op
+        self.assertEqual(defopt.run(main, argv=['1']), (1,))
+
+    def test_union_operator_doc_list(self):
+        if not hasattr(types, "UnionType"):
+            raise unittest.SkipTest("A|B-style unions not supported")
+        def main(op):
+            """:param list[int]|None op: op"""
+            return op
+        self.assertEqual(defopt.run(main, argv=['--op', '1', '2']), [1, 2])
+
+    def test_union_operator_doc_one_item_tuple(self):
+        if not hasattr(types, "UnionType"):
+            raise unittest.SkipTest("A|B-style unions not supported")
+        def main(op):
+            """:param tuple[int]|None op: op"""
+            return op
+        self.assertEqual(defopt.run(main, argv=['1']), (1,))
+
+    def test_union_operator_doc_multiple_item_tuple(self):
+        if not hasattr(types, "UnionType"):
+            raise unittest.SkipTest("A|B-style unions not supported")
+        def main(op):
+            """:param tuple[str,str]|None op: op"""
+            return op
+        self.assertEqual(defopt.run(main, argv=['a', 'b']), ('a', 'b'))
+
+    def test_multiple_item_optional_tuple_none_parser(self):
+        def main(op):
+            """:param typing.Optional[typing.Tuple[int, int]] op: op"""
+            return op
+        def _parse_none(i):
+            if i.lower() == 'none':
+                return None
+            else:
+                raise ValueError('{} is not a valid None string'.format(i))
+        with self.assertRaises(ValueError):
+            defopt.run(main, argv=['1', '2'],
+                       parsers={type(None): _parse_none})
+
+    def test_one_item_optional_tuple_none_parser(self):
+        # As long as there is not support for trying the NoneType parser
+        # before the tuple parser in the case of a one-item optional tuple,
+        # this case should also raise an error. See comments on GH115 for more
+        # details.
+        def main(op):
+            """:param typing.Optional[typing.Tuple[str]] op: op"""
+            return op
+        def _parse_none(i):
+            if i.lower() == 'none':
+                return None
+            else:
+                raise ValueError('{} is not a valid None string'.format(i))
+        with self.assertRaises(ValueError):
+            defopt.run(main, argv=['1'], parsers={type(None): _parse_none})
+
+
 class TestLiteral(unittest.TestCase):
     def test_literal(self):
         def main(foo):
@@ -1016,11 +1128,6 @@ class TestAnnotations(unittest.TestCase):
         self.assertEqual(defopt._get_type_from_hint(typing.Sequence[int]),
                          typing.List[int])
 
-    def test_optional(self):
-        self.assertEqual(
-            defopt._get_type_from_hint(typing.Optional[typing.List[int]]),
-            typing.List[int])
-
     def test_conflicting(self):
         def foo(bar: int):
             """:type bar: float"""
@@ -1155,6 +1262,33 @@ class TestHelp(unittest.TestCase):
         self.assert_in_help('%(prog)s', foo, '')
         self.assert_not_in_help('%%', foo, '')
 
+    def test_hyperlink_plaintext(self):
+        def foo():
+            """This site https://www.python.org/ is cool"""
+        self.assert_in_help(
+            "This site https://www.python.org/ is cool",
+            foo,
+            ""
+        )
+
+    def test_hyperlink_target_embedded(self):
+        def foo():
+            """`This site <https://www.python.org/>`_ is cool"""
+        self.assert_in_help(
+            "This site (https://www.python.org/) is cool",
+            foo,
+            ""
+        )
+
+    def test_hyperlink_target_separated(self):
+        def foo():
+            """This site_ is cool
+
+            .. _site: https://www.python.org/
+            """
+        self.assert_in_help("This site is cool", foo, "")
+        self.assert_not_in_help("https://www.python.org/", foo, "")
+
     def test_rst_ansi(self):
         def foo():
             """**bold** *italic* `underlined`"""
@@ -1387,28 +1521,34 @@ class TestRunAny(unittest.TestCase):  # TODO: Reuse TestExamples.
     def test_lists_cli(self):
         output = self._run_any(
             'examples.lists.main', ['2', '--numbers', '1.2', '3.4'])
-        self.assertEqual(output, b'[2.4, 6.8]\n')
+        self.assertEqual(output, '[2.4, 6.8]\n')
         output = self._run_any(
-            'examples.lists.main', ['--numbers', '1.2', '3.4', '--', '2'])
-        self.assertEqual(output, b'[2.4, 6.8]\n')
+            'examples.lists:main', ['--numbers', '1.2', '3.4', '--', '2'])
+        self.assertEqual(output, '[2.4, 6.8]\n')
 
     def test_failed_imports(self):
-        with self.assertRaises(subprocess.CalledProcessError) as error:
-            self._run_any('does-not-exist', [])
-        self.assertIn(b"Failed to locate 'does-not-exist'",
-                      error.exception.output)
-        with TemporaryDirectory() as tmpdir:
-            Path(tmpdir, 'bad_module.py').write_text('1+')
-            with self.assertRaises(subprocess.CalledProcessError) as error:
-                self._run_any('bad_module', [], cwd=tmpdir)
-            self.assertRegex(error.exception.output,
-                             br'SyntaxError: invalid syntax\n\Z')
+        with self.assertRaises((ImportError, subprocess.CalledProcessError)):
+            self._run_any('does_not_exist', [])
+
+    @staticmethod
+    def _target(command, argv):
+        sys.argv[1:] = [command, *argv]  # Only executed in a subprocess.
+        buf = StringIO()
+        with contextlib.redirect_stdout(buf), contextlib.redirect_stderr(buf):
+            runpy.run_module('defopt', run_name='__main__', alter_sys=True)
+        return buf.getvalue()
 
     def _run_any(self, command, argv, **kwargs):
-        output = subprocess.check_output(
-            [sys.executable, '-m', 'defopt', command] + argv,
-            stderr=subprocess.STDOUT, **kwargs)
-        return output.replace(b'\r\n', b'\n')
+        if sys.version_info >= (3, 7):  # multiprocessing for proper coverage.
+            with ProcessPoolExecutor(mp_context=mp.get_context('spawn')) \
+                    as executor:
+                fut = executor.submit(self._target, command, argv)
+            return fut.result().replace('\r\n', '\n')
+        else:  # mp_context is not available.
+            output = subprocess.check_output(
+                [sys.executable, '-m', 'defopt', command] + argv,
+                stderr=subprocess.STDOUT, universal_newlines=True, **kwargs)
+            return output.replace('\r\n', '\n')
 
 
 class TestDefaultsPreserved(unittest.TestCase):
@@ -1420,9 +1560,18 @@ class TestDefaultsPreserved(unittest.TestCase):
 
 
 class TestStyle(unittest.TestCase):
-    def test_line_length(self):
+    def _iter_stripped_lines(self):
         for path in [defopt.__file__, __file__]:
             with tokenize.open(path) as src:
                 for i, line in enumerate(src, 1):
-                    if len(line) > 80:  # 79 chars + newline.
-                        self.fail('{}:{} is too long'.format(path, i))
+                    yield '{}:{}'.format(path, i), line.rstrip('\n')
+
+    def test_line_length(self):
+        for name, line in self._iter_stripped_lines():
+            if len(line) > 79:
+                self.fail('{} is too long'.format(name))
+
+    def test_trailing_whitespace(self):
+        for name, line in self._iter_stripped_lines():
+            if line and line[-1].isspace():
+                self.fail('{} has trailing whitespace'.format(name))
